from daaja.eda.easy_data_augmentor import EasyDataAugmentor
from daaja.eda.randam_delete_augmentor import RandamDeleteAugmentor
from daaja.eda.randam_insert_augmentor import RandamInsertAugmentor
from daaja.eda.randam_swap_augmentor import RandamSwapAugmentor
from daaja.eda.synonym_replace_augmentor import SynonymReplaceAugmentor



import random
from typing import List, Tuple

from daaja.ner_sda.labelwise_token_replacement_augmentor import \
    LabelwiseTokenReplacementAugmentor
from daaja.ner_sda.mention_replacement_augmentor import \
    MentionReplacementAugmentor
from daaja.ner_sda.shuffle_within_segments_augmentor import \
    ShuffleWithinSegmentsAugmentor
from daaja.ner_sda.synonym_replacement_augmentor import \
    SynonymReplacementAugmentor
from daaja.ner_sda.utils import get_entity_dict, get_token2prob_in_label
from daaja.resouces import Resouces


class SimpleDataAugmentationforNER:
    def __init__(self,
                 tokens_list: List[List[str]],
                 labels_list: List[List[str]],
                 p_power: float,
                 p_lwtr: float,
                 p_mr: float,
                 p_sis: float,
                 p_sr: float,
                 num_aug: int):
        # setup
        resouces = Resouces()
        entity_dict = get_entity_dict(tokens_list, labels_list)
        token_and_prob_in_label = get_token2prob_in_label(tokens_list, labels_list, p_power=p_power)

        # augmentors
        self.labelwise_token_replacement_augmentor = LabelwiseTokenReplacementAugmentor(token_and_prob_in_label, p=p_lwtr, resouces=resouces)
        self.mention_replacement_augmentor = MentionReplacementAugmentor(entity_dict=entity_dict, p=p_mr)
        self.shuffle_within_segments_augmentor = ShuffleWithinSegmentsAugmentor(p=p_sis)
        self.synonym_replacement_augmentor = SynonymReplacementAugmentor(token_and_prob_in_label=token_and_prob_in_label, p=p_sr, resouces=resouces)

        # params
        self.num_aug = num_aug

    def augments(self, tokens: List[str], labels: List[str]) -> Tuple[List[List[str]], List[List[str]]]:
        augmented_tokens_and_labels = []
        num_new_per_technique = int(self.num_aug / 4) + 1

        # Label-wise token replacement (LwTR)
        if self.labelwise_token_replacement_augmentor.p > 0:
            for _ in range(num_new_per_technique):
                aug_tokens, aug_labels = self.labelwise_token_replacement_augmentor.augment(tokens, labels)
                augmented_tokens_and_labels.append([aug_tokens, aug_labels])

        # Synonym replacement (SR)
        if self.labelwise_token_replacement_augmentor.p > 0:
            for _ in range(num_new_per_technique):
                aug_tokens, aug_labels = self.synonym_replacement_augmentor.augment(tokens, labels)
                augmented_tokens_and_labels.append([aug_tokens, aug_labels])

        # Mention replacement (MR)
        if self.labelwise_token_replacement_augmentor.p > 0:
            for _ in range(num_new_per_technique):
                aug_tokens, aug_labels = self.mention_replacement_augmentor.augment(tokens, labels)
                augmented_tokens_and_labels.append([aug_tokens, aug_labels])

        # Shuffle within segments (SiS)
        if self.labelwise_token_replacement_augmentor.p > 0:
            for _ in range(num_new_per_technique):
                aug_tokens, aug_labels = self.shuffle_within_segments_augmentor.augment(tokens, labels)
                augmented_tokens_and_labels.append([aug_tokens, aug_labels])

        random.shuffle(augmented_tokens_and_labels)
        if self.num_aug >= 1:
            augmented_tokens_and_labels = augmented_tokens_and_labels[:self.num_aug]
        else:
            keep_prob = self.num_aug / len(augmented_tokens_and_labels)
            augmented_tokens_and_labels = [s for s in augmented_tokens_and_labels if random.uniform(0, 1) < keep_prob]

        # Append the original sentence
        augmented_tokens_and_labels.append([tokens, labels])
        augmented_tokens_list = [tokens_list for tokens_list, _ in augmented_tokens_and_labels]
        augmented_labels_list = [labels_list for _, labels_list in augmented_tokens_and_labels]
        return augmented_tokens_list, augmented_labels_list



import random
from typing import List

from daaja.augmentor import Augmentor
from daaja.tokenizer import Tokenizer


class RandamSwapAugmentor(Augmentor):
    def __init__(self,
                 alpha: float = 0.1,
                 tokenizer: Tokenizer = Tokenizer()) -> None:
        self.alpha = alpha
        self.tokenizer = tokenizer

    def augment(self, sentence: str) -> str:
        tokens, _ = self.tokenizer.tokenize(sentence)
        n = max(1, int(self.alpha * len(tokens)))
        converted_tokens = self.random_swap(tokens, n)
        return "".join(converted_tokens)

    def random_swap(self, tokens: List[str], n: int) -> List[str]:
        """Randomly swap two words in the sentence n times

        Args:
            tokens (List[str]): sentence
            n (int): Number of swapping times

        Returns:
            List[str]: swaped sentence
        """
        converted_tokens = tokens.copy()
        for _ in range(n):
            converted_tokens = self._swap_word(converted_tokens)
        return converted_tokens

    def _swap_word(self, tokens: List[str]) -> List[str]:
        random_idx_1 = random.randint(0, len(tokens) - 1)
        random_idx_2 = random_idx_1
        cnt = 0
        while random_idx_1 == random_idx_2:
            random_idx_2 = random.randint(0, len(tokens) - 1)
            cnt += 1
            if cnt > 3:
                return tokens
        tokens[random_idx_1], tokens[random_idx_2] = tokens[random_idx_2], tokens[random_idx_1]
        return tokens



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['daaja', 'daaja.eda', 'daaja.ner_sda']

package_data = \
{'': ['*']}

install_requires = \
['SudachiDict-core>=20211220,<20211221',
 'SudachiPy>=0.6.3,<0.7.0',
 'pandas>=1.4.0,<2.0.0',
 'requests>=2.27.1,<3.0.0']

setup_kwargs = {
    'name': 'daaja',
    'version': '0.0.1',
    'description': '',
    'long_description': None,
    'author': 'Koga Kobayashi',
    'author_email': 'kajyuuen@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import random
from typing import List

from daaja.eda.randam_delete_augmentor import RandamDeleteAugmentor
from daaja.eda.randam_insert_augmentor import RandamInsertAugmentor
from daaja.eda.randam_swap_augmentor import RandamSwapAugmentor
from daaja.eda.synonym_replace_augmentor import SynonymReplaceAugmentor
from daaja.resouces import Resouces
from daaja.tokenizer import Tokenizer


class EasyDataAugmentor:
    def __init__(self, alpha_sr: float, alpha_ri: float, alpha_rs: float, p_rd: float, num_aug: int):
        resouces = Resouces()
        self.tokenizer = Tokenizer()

        # augmentors
        self.synonym_replace_augmentor = SynonymReplaceAugmentor(alpha=alpha_sr, resouces=resouces, tokenizer=self.tokenizer)
        self.randam_insert_augmentor = RandamInsertAugmentor(alpha=alpha_ri, resouces=resouces, tokenizer=self.tokenizer)
        self.randam_swap_augmentor = RandamSwapAugmentor(alpha=alpha_rs, tokenizer=self.tokenizer)
        self.randam_delete_augmentor = RandamDeleteAugmentor(p=p_rd, tokenizer=self.tokenizer)

        # params
        self.num_aug = num_aug

    def augments(self, sentence: str) -> List[str]:
        tokens, selected_tokens = self.tokenizer.tokenize(sentence)
        num_tokens = len(tokens)
        augmented_sentences = []
        num_new_per_technique = int(self.num_aug / 4) + 1

        # Synonym Replace
        if self.synonym_replace_augmentor.alpha > 0:
            n_sr = max(1, int(self.synonym_replace_augmentor.alpha * num_tokens))
            for _ in range(num_new_per_technique):
                aug_tokens = self.synonym_replace_augmentor.synonym_replace(tokens, selected_tokens, n_sr)
                augmented_sentences.append("".join(aug_tokens))

        # Random Insert
        if self.randam_insert_augmentor.alpha > 0:
            n_ri = max(1, int(self.randam_insert_augmentor.alpha * num_tokens))
            for _ in range(num_new_per_technique):
                aug_tokens = self.randam_insert_augmentor.random_insert(tokens, selected_tokens, n_ri)
                augmented_sentences.append("".join(aug_tokens))

        # Random Swap
        if self.randam_swap_augmentor.alpha > 0:
            n_ri = max(1, int(self.randam_swap_augmentor.alpha * num_tokens))
            for _ in range(num_new_per_technique):
                aug_tokens = self.randam_swap_augmentor.random_swap(tokens, n_ri)
                augmented_sentences.append("".join(aug_tokens))

        # Random Delete
        if self.randam_delete_augmentor.p > 0:
            for _ in range(num_new_per_technique):
                p = self.randam_delete_augmentor.p
                aug_tokens = self.randam_delete_augmentor.random_delete(tokens, p)
                augmented_sentences.append("".join(aug_tokens))

        random.shuffle(augmented_sentences)
        if self.num_aug >= 1:
            augmented_sentences = augmented_sentences[:self.num_aug]
        else:
            keep_prob = self.num_aug / len(augmented_sentences)
            augmented_sentences = [s for s in augmented_sentences if random.uniform(0, 1) < keep_prob]

        # Append the original sentence
        augmented_sentences.append(sentence)
        return augmented_sentences



import random
from typing import List

from daaja.augmentor import Augmentor
from daaja.tokenizer import Tokenizer


class RandamDeleteAugmentor(Augmentor):
    def __init__(self,
                 p: float = 0.05,
                 tokenizer: Tokenizer = Tokenizer()) -> None:
        self.p = p
        self.tokenizer = tokenizer

    def augment(self, sentence: str) -> str:
        tokens, _ = self.tokenizer.tokenize(sentence)
        converted_tokens = self.random_delete(tokens, self.p)
        return "".join(converted_tokens)

    def random_delete(self, tokens: List[str], p: float) -> List[str]:
        """Randomly delete tokens from the sentence with probability p

        Args:
            tokens (List[str]): Sentence
            p (float): probability

        Returns:
            List[str]: tokens with n tokens removed
        """
        if len(tokens) == 1:
            return tokens

        converted_tokens = []
        for token in tokens:
            r = random.uniform(0, 1)
            if r < p:
                continue
            converted_tokens.append(token)

        if len(converted_tokens) == 0:
            random_idx = random.randint(0, len(tokens) - 1)
            return [tokens[random_idx]]

        return converted_tokens



from daaja.ner_sda.labelwise_token_replacement_augmentor import \
    LabelwiseTokenReplacementAugmentor
from daaja.ner_sda.mention_replacement_augmentor import \
    MentionReplacementAugmentor
from daaja.ner_sda.shuffle_within_segments_augmentor import \
    ShuffleWithinSegmentsAugmentor
from daaja.ner_sda.simple_data_augmentation_for_ner import \
    SimpleDataAugmentationforNER
from daaja.ner_sda.synonym_replacement_augmentor import \
    SynonymReplacementAugmentor
